@@ -1,15 +1,15 @@
 /* \summary: Solaris DLT_IPNET printer */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
+#define ND_LONGJMP_FROM_TCHECK
 #include "netdissect.h"
 #include "extract.h"
 
-static const char tstr[] = "[|ipnet]";
 
 typedef struct ipnet_hdr {
 	nd_uint8_t	iph_version;
@@ -33,32 +33,28 @@ static const struct tok ipnet_values[] = {
 	{ 0,			NULL }
 };
 
-static inline void
+static void
 ipnet_hdr_print(netdissect_options *ndo, const u_char *bp, u_int length)
 {
 	const ipnet_hdr_t *hdr;
 	hdr = (const ipnet_hdr_t *)bp;
 
-	ND_TCHECK(*hdr);
-	ND_PRINT((ndo, "%d > %d", EXTRACT_32BITS(hdr->iph_zsrc),
-		  EXTRACT_32BITS(hdr->iph_zdst)));
+	ND_PRINT("%u > %u", GET_BE_U_4(hdr->iph_zsrc),
+		  GET_BE_U_4(hdr->iph_zdst));
 
 	if (!ndo->ndo_qflag) {
-		ND_PRINT((ndo,", family %s (%d)",
+		ND_PRINT(", family %s (%u)",
                           tok2str(ipnet_values, "Unknown",
-                                  EXTRACT_8BITS(&hdr->iph_family)),
-                          EXTRACT_8BITS(&hdr->iph_family)));
+                                  GET_U_1(hdr->iph_family)),
+                          GET_U_1(hdr->iph_family));
         } else {
-		ND_PRINT((ndo,", %s",
+		ND_PRINT(", %s",
                           tok2str(ipnet_values,
                                   "Unknown Ethertype (0x%04x)",
-				  EXTRACT_8BITS(&hdr->iph_family))));
+				  GET_U_1(hdr->iph_family)));
         }
 
-	ND_PRINT((ndo, ", length %u: ", length));
-	return;
-trunc:
-	ND_PRINT((ndo, " %s", tstr));
+	ND_PRINT(", length %u: ", length);
 }
 
 static void
@@ -66,8 +62,8 @@ ipnet_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen
 {
 	const ipnet_hdr_t *hdr;
 
-	if (caplen < sizeof(ipnet_hdr_t))
-		goto trunc;
+	ND_TCHECK_LEN(p, sizeof(ipnet_hdr_t));
+	ndo->ndo_ll_hdr_len += sizeof(ipnet_hdr_t);
 
 	if (ndo->ndo_eflag)
 		ipnet_hdr_print(ndo, p, length);
@@ -77,8 +73,7 @@ ipnet_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen
 	hdr = (const ipnet_hdr_t *)p;
 	p += sizeof(ipnet_hdr_t);
 
-	ND_TCHECK2(hdr->iph_family, 1);
-	switch (EXTRACT_8BITS(&hdr->iph_family)) {
+	switch (GET_U_1(hdr->iph_family)) {
 
 	case IPH_AF_INET:
 	        ip_print(ndo, p, length);
@@ -97,9 +92,6 @@ ipnet_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen
 			ND_DEFAULTPRINT(p, caplen);
 		break;
 	}
-	return;
-trunc:
-	ND_PRINT((ndo, " %s", tstr));
 }
 
 /*
@@ -108,20 +100,11 @@ trunc:
  * 'h->len' is the length of the packet off the wire, and 'h->caplen'
  * is the number of bytes actually captured.
  */
-u_int
+void
 ipnet_if_print(netdissect_options *ndo,
                const struct pcap_pkthdr *h, const u_char *p)
 {
+	ndo->ndo_protocol = "ipnet";
 	ipnet_print(ndo, p, h->len, h->caplen);
-
-	return (sizeof(ipnet_hdr_t));
 }
-
-/*
- * Local Variables:
- * c-style: whitesmith
- * c-basic-offset: 8
- * End:
- */
-
 #endif /* DLT_IPNET */
