@@ -24,10 +24,10 @@
 /* \summary: IPv6 Open Shortest Path First (OSPFv3) printer */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
 #include <string.h>
 
@@ -95,7 +95,7 @@
 #define	RLA_FLAG_E	0x02
 #define	RLA_FLAG_V	0x04
 #define	RLA_FLAG_W	0x08
-#define RLA_FLAG_N      0x10
+#define	RLA_FLAG_Nt	0x10
 
 /* lsa_prefix options */
 #define LSA_PREFIX_OPT_NU 0x01
@@ -103,6 +103,7 @@
 #define LSA_PREFIX_OPT_MC 0x04
 #define LSA_PREFIX_OPT_P  0x08
 #define LSA_PREFIX_OPT_DN 0x10
+#define LSA_PREFIX_OPT_N  0x20
 
 /* sla_tosmetric breakdown	*/
 #define	SLA_MASK_TOS		0x7f000000
@@ -118,27 +119,27 @@
 #define OSPF6_AT_HDRLEN             16U
 #define OSPF6_AUTH_TYPE_HMAC        0x0001
 
-typedef uint32_t rtrid_t;
+typedef nd_uint32_t rtrid_t;
 
 /* link state advertisement header */
 struct lsa6_hdr {
-    uint16_t ls_age;
-    uint16_t ls_type;
+    nd_uint16_t ls_age;
+    nd_uint16_t ls_type;
     rtrid_t ls_stateid;
     rtrid_t ls_router;
-    uint32_t ls_seq;
-    uint16_t ls_chksum;
-    uint16_t ls_length;
+    nd_uint32_t ls_seq;
+    nd_uint16_t ls_chksum;
+    nd_uint16_t ls_length;
 };
 
 /* Length of an IPv6 address, in bytes. */
 #define IPV6_ADDR_LEN_BYTES (128/8)
 
 struct lsa6_prefix {
-    uint8_t lsa_p_len;
-    uint8_t lsa_p_opt;
-    uint16_t lsa_p_metric;
-    uint8_t lsa_p_prefix[IPV6_ADDR_LEN_BYTES]; /* maximum length */
+    nd_uint8_t lsa_p_len;
+    nd_uint8_t lsa_p_opt;
+    nd_uint16_t lsa_p_metric;
+    nd_byte lsa_p_prefix[IPV6_ADDR_LEN_BYTES]; /* maximum length */
 };
 
 /* link state advertisement */
@@ -150,36 +151,36 @@ struct lsa6 {
 	/* Router links advertisements */
 	struct {
 	    union {
-		uint8_t flg;
-		uint32_t opt;
+		nd_uint8_t flg;
+		nd_uint32_t opt;
 	    } rla_flgandopt;
 #define rla_flags	rla_flgandopt.flg
 #define rla_options	rla_flgandopt.opt
 	    struct rlalink6 {
-		uint8_t link_type;
-		uint8_t link_zero[1];
-		uint16_t link_metric;
-		uint32_t link_ifid;
-		uint32_t link_nifid;
+		nd_uint8_t link_type;
+		nd_byte link_zero;
+		nd_uint16_t link_metric;
+		nd_uint32_t link_ifid;
+		nd_uint32_t link_nifid;
 		rtrid_t link_nrtid;
 	    } rla_link[1];		/* may repeat	*/
 	} un_rla;
 
 	/* Network links advertisements */
 	struct {
-	    uint32_t nla_options;
+	    nd_uint32_t nla_options;
 	    rtrid_t nla_router[1];	/* may repeat	*/
 	} un_nla;
 
 	/* Inter Area Prefix LSA */
 	struct {
-	    uint32_t inter_ap_metric;
+	    nd_uint32_t inter_ap_metric;
 	    struct lsa6_prefix inter_ap_prefix[1];
 	} un_inter_ap;
 
 	/* AS external links advertisements */
 	struct {
-	    uint32_t asla_metric;
+	    nd_uint32_t asla_metric;
 	    struct lsa6_prefix asla_prefix[1];
 	    /* some optional fields follow */
 	} un_asla;
@@ -187,14 +188,14 @@ struct lsa6 {
 #if 0
 	/* Summary links advertisements */
 	struct {
-	    struct in_addr sla_mask;
-	    uint32_t sla_tosmetric[1];	/* may repeat	*/
+	    nd_ipv4     sla_mask;
+	    nd_uint32_t sla_tosmetric[1];	/* may repeat	*/
 	} un_sla;
 
 	/* Multicast group membership */
 	struct mcla {
-	    uint32_t mcla_vtype;
-	    struct in_addr mcla_vid;
+	    nd_uint32_t mcla_vtype;
+	    nd_ipv4     mcla_vid;
 	} un_mcla[1];
 #endif
 
@@ -203,20 +204,20 @@ struct lsa6 {
 	/* Link LSA */
 	struct llsa {
 	    union {
-		uint8_t pri;
-		uint32_t opt;
+		nd_uint8_t pri;
+		nd_uint32_t opt;
 	    } llsa_priandopt;
 #define llsa_priority	llsa_priandopt.pri
 #define llsa_options	llsa_priandopt.opt
-	    struct in6_addr llsa_lladdr;
-	    uint32_t llsa_nprefix;
+	    nd_ipv6	llsa_lladdr;
+	    nd_uint32_t llsa_nprefix;
 	    struct lsa6_prefix llsa_prefix[1];
 	} un_llsa;
 
 	/* Intra-Area-Prefix */
 	struct {
-	    uint16_t intra_ap_nprefix;
-	    uint16_t intra_ap_lstype;
+	    nd_uint16_t intra_ap_nprefix;
+	    nd_uint16_t intra_ap_lstype;
 	    rtrid_t intra_ap_lsid;
 	    rtrid_t intra_ap_rtid;
 	    struct lsa6_prefix intra_ap_prefix[1];
@@ -228,14 +229,14 @@ struct lsa6 {
  * the main header
  */
 struct ospf6hdr {
-    uint8_t ospf6_version;
-    uint8_t ospf6_type;
-    uint16_t ospf6_len;
+    nd_uint8_t ospf6_version;
+    nd_uint8_t ospf6_type;
+    nd_uint16_t ospf6_len;
     rtrid_t ospf6_routerid;
     rtrid_t ospf6_areaid;
-    uint16_t ospf6_chksum;
-    uint8_t ospf6_instanceid;
-    uint8_t ospf6_rsvd;
+    nd_uint16_t ospf6_chksum;
+    nd_uint8_t ospf6_instanceid;
+    nd_uint8_t ospf6_rsvd;
 };
 
 /*
@@ -246,15 +247,15 @@ struct ospf6hdr {
 
 /* Hello packet */
 struct hello6 {
-    uint32_t hello_ifid;
+    nd_uint32_t hello_ifid;
     union {
-	uint8_t pri;
-	uint32_t opt;
+	nd_uint8_t pri;
+	nd_uint32_t opt;
     } hello_priandopt;
 #define hello_priority	hello_priandopt.pri
 #define hello_options	hello_priandopt.opt
-    uint16_t hello_helloint;
-    uint16_t hello_deadint;
+    nd_uint16_t hello_helloint;
+    nd_uint16_t hello_deadint;
     rtrid_t hello_dr;
     rtrid_t hello_bdr;
     rtrid_t hello_neighbor[1]; /* may repeat	*/
@@ -262,29 +263,28 @@ struct hello6 {
 
 /* Database Description packet */
 struct dd6 {
-    uint32_t db_options;
-    uint16_t db_mtu;
-    uint8_t db_mbz;
-    uint8_t db_flags;
-    uint32_t db_seq;
+    nd_uint32_t db_options;
+    nd_uint16_t db_mtu;
+    nd_uint8_t db_mbz;
+    nd_uint8_t db_flags;
+    nd_uint32_t db_seq;
     struct lsa6_hdr db_lshdr[1]; /* may repeat	*/
 };
 
 /* Link State Request */
 struct lsr6 {
-    uint16_t ls_mbz;
-    uint16_t ls_type;
+    nd_uint16_t ls_mbz;
+    nd_uint16_t ls_type;
     rtrid_t ls_stateid;
     rtrid_t ls_router;
 };
 
 /* Link State Update */
 struct lsu6 {
-    uint32_t lsu_count;
+    nd_uint32_t lsu_count;
     struct lsa6 lsu_lsa[1]; /* may repeat	*/
 };
 
-static const char tstr[] = " [|ospf3]";
 
 static const struct tok ospf6_option_values[] = {
 	{ OSPF6_OPTION_V6,	"V6" },
@@ -303,8 +303,8 @@ static const struct tok ospf6_rla_flag_values[] = {
 	{ RLA_FLAG_B,		"ABR" },
 	{ RLA_FLAG_E,		"External" },
 	{ RLA_FLAG_V,		"Virtual-Link Endpoint" },
-	{ RLA_FLAG_W,		"Wildcard Receiver" },
-        { RLA_FLAG_N,           "NSSA Translator" },
+	{ RLA_FLAG_W,		"Deprecated" },
+	{ RLA_FLAG_Nt,		"NSSA Translator" },
 	{ 0,			NULL }
 };
 
@@ -363,6 +363,7 @@ static const struct tok ospf6_lsa_prefix_option_values[] = {
         { LSA_PREFIX_OPT_MC, "Deprecated" },
         { LSA_PREFIX_OPT_P, "Propagate" },
         { LSA_PREFIX_OPT_DN, "Down" },
+        { LSA_PREFIX_OPT_N, "N-bit" },
 	{ 0, NULL }
 };
 
@@ -373,31 +374,31 @@ static const struct tok ospf6_auth_type_str[] = {
 
 static void
 ospf6_print_ls_type(netdissect_options *ndo,
-                    register u_int ls_type, register const rtrid_t *ls_stateid)
+                    u_int ls_type, const rtrid_t *ls_stateid)
 {
-        ND_PRINT((ndo, "\n\t    %s LSA (%d), %s Scope%s, LSA-ID %s",
+        ND_PRINT("\n\t    %s LSA (%u), %s Scope%s, LSA-ID %s",
                tok2str(ospf6_lsa_values, "Unknown", ls_type & LS_TYPE_MASK),
                ls_type & LS_TYPE_MASK,
                tok2str(ospf6_ls_scope_values, "Unknown", ls_type & LS_SCOPE_MASK),
                ls_type &0x8000 ? ", transitive" : "", /* U-bit */
-               ipaddr_string(ndo, ls_stateid)));
+               GET_IPADDR_STRING((const u_char *)ls_stateid));
 }
 
 static int
 ospf6_print_lshdr(netdissect_options *ndo,
-                  register const struct lsa6_hdr *lshp, const u_char *dataend)
+                  const struct lsa6_hdr *lshp, const u_char *dataend)
 {
 	if ((const u_char *)(lshp + 1) > dataend)
 		goto trunc;
-	ND_TCHECK(lshp->ls_length);	/* last field of struct lsa6_hdr */
 
-	ND_PRINT((ndo, "\n\t  Advertising Router %s, seq 0x%08x, age %us, length %u",
-               ipaddr_string(ndo, &lshp->ls_router),
-               EXTRACT_32BITS(&lshp->ls_seq),
-               EXTRACT_16BITS(&lshp->ls_age),
-               EXTRACT_16BITS(&lshp->ls_length)-(u_int)sizeof(struct lsa6_hdr)));
+	ND_PRINT("\n\t  Advertising Router %s, seq 0x%08x, age %us, length %zu",
+		 GET_IPADDR_STRING(lshp->ls_router),
+		 GET_BE_U_4(lshp->ls_seq),
+		 GET_BE_U_2(lshp->ls_age),
+		 GET_BE_U_2(lshp->ls_length)-sizeof(struct lsa6_hdr));
 
-	ospf6_print_ls_type(ndo, EXTRACT_16BITS(&lshp->ls_type), &lshp->ls_stateid);
+	ospf6_print_ls_type(ndo, GET_BE_U_2(lshp->ls_type),
+			    &lshp->ls_stateid);
 
 	return (0);
 trunc:
@@ -410,31 +411,30 @@ ospf6_print_lsaprefix(netdissect_options *ndo,
 {
 	const struct lsa6_prefix *lsapp = (const struct lsa6_prefix *)tptr;
 	u_int wordlen;
-	struct in6_addr prefix;
+	nd_ipv6 prefix;
 
 	if (lsa_length < sizeof (*lsapp) - IPV6_ADDR_LEN_BYTES)
 		goto trunc;
 	lsa_length -= sizeof (*lsapp) - IPV6_ADDR_LEN_BYTES;
-	ND_TCHECK2(*lsapp, sizeof (*lsapp) - IPV6_ADDR_LEN_BYTES);
-	wordlen = (lsapp->lsa_p_len + 31) / 32;
-	if (wordlen * 4 > sizeof(struct in6_addr)) {
-		ND_PRINT((ndo, " bogus prefixlen /%d", lsapp->lsa_p_len));
+	ND_TCHECK_LEN(lsapp, sizeof(*lsapp) - IPV6_ADDR_LEN_BYTES);
+	wordlen = (GET_U_1(lsapp->lsa_p_len) + 31) / 32;
+	if (wordlen * 4 > sizeof(nd_ipv6)) {
+		ND_PRINT(" bogus prefixlen /%u", GET_U_1(lsapp->lsa_p_len));
 		goto trunc;
 	}
 	if (lsa_length < wordlen * 4)
 		goto trunc;
 	lsa_length -= wordlen * 4;
-	ND_TCHECK2(lsapp->lsa_p_prefix, wordlen * 4);
-	memset(&prefix, 0, sizeof(prefix));
-	memcpy(&prefix, lsapp->lsa_p_prefix, wordlen * 4);
-	ND_PRINT((ndo, "\n\t\t%s/%d", ip6addr_string(ndo, &prefix),
-		lsapp->lsa_p_len));
-        if (lsapp->lsa_p_opt) {
-            ND_PRINT((ndo, ", Options [%s]",
+	memset(prefix, 0, sizeof(prefix));
+	GET_CPY_BYTES(prefix, lsapp->lsa_p_prefix, wordlen * 4);
+	ND_PRINT("\n\t\t%s/%u", ip6addr_string(ndo, prefix), /* local buffer, not packet data; don't use GET_IP6ADDR_STRING() */
+		 GET_U_1(lsapp->lsa_p_len));
+        if (GET_U_1(lsapp->lsa_p_opt)) {
+            ND_PRINT(", Options [%s]",
                    bittok2str(ospf6_lsa_prefix_option_values,
-                              "none", lsapp->lsa_p_opt)));
+                              "none", GET_U_1(lsapp->lsa_p_opt)));
         }
-        ND_PRINT((ndo, ", metric %u", EXTRACT_16BITS(&lsapp->lsa_p_metric)));
+        ND_PRINT(", metric %u", GET_BE_U_2(lsapp->lsa_p_metric));
 	return sizeof(*lsapp) - IPV6_ADDR_LEN_BYTES + wordlen * 4;
 
 trunc:
@@ -447,32 +447,31 @@ trunc:
  */
 static int
 ospf6_print_lsa(netdissect_options *ndo,
-                register const struct lsa6 *lsap, const u_char *dataend)
+                const struct lsa6 *lsap, const u_char *dataend)
 {
-	register const struct rlalink6 *rlp;
+	const struct rlalink6 *rlp;
 #if 0
-	register const struct tos_metric *tosp;
+	const struct tos_metric *tosp;
 #endif
-	register const rtrid_t *ap;
+	const rtrid_t *ap;
 #if 0
-	register const struct aslametric *almp;
-	register const struct mcla *mcp;
+	const struct aslametric *almp;
+	const struct mcla *mcp;
 #endif
-	register const struct llsa *llsap;
-	register const struct lsa6_prefix *lsapp;
+	const struct llsa *llsap;
+	const struct lsa6_prefix *lsapp;
 #if 0
-	register const uint32_t *lp;
+	const uint32_t *lp;
 #endif
-	register u_int prefixes;
-	register int bytelen;
-	register u_int length, lsa_length;
+	u_int prefixes;
+	int bytelen;
+	u_int length, lsa_length;
 	uint32_t flags32;
 	const uint8_t *tptr;
 
 	if (ospf6_print_lshdr(ndo, &lsap->ls_hdr, dataend))
 		return (1);
-	ND_TCHECK(lsap->ls_hdr.ls_length);
-        length = EXTRACT_16BITS(&lsap->ls_hdr.ls_length);
+        length = GET_BE_U_2(lsap->ls_hdr.ls_length);
 
 	/*
 	 * The LSA length includes the length of the header;
@@ -481,61 +480,60 @@ ospf6_print_lsa(netdissect_options *ndo,
 	 * header.
 	 */
         if (length < sizeof(struct lsa6_hdr) || (const u_char *)lsap + length > dataend)
-        	return (1);
+		return (1);
         lsa_length = length - sizeof(struct lsa6_hdr);
         tptr = (const uint8_t *)lsap+sizeof(struct lsa6_hdr);
 
-	switch (EXTRACT_16BITS(&lsap->ls_hdr.ls_type)) {
+	switch (GET_BE_U_2(lsap->ls_hdr.ls_type)) {
 	case LS_TYPE_ROUTER | LS_SCOPE_AREA:
 		if (lsa_length < sizeof (lsap->lsa_un.un_rla.rla_options))
 			return (1);
 		lsa_length -= sizeof (lsap->lsa_un.un_rla.rla_options);
-		ND_TCHECK(lsap->lsa_un.un_rla.rla_options);
-		ND_PRINT((ndo, "\n\t      Options [%s]",
+		ND_PRINT("\n\t      Options [%s]",
 		          bittok2str(ospf6_option_values, "none",
-		          EXTRACT_32BITS(&lsap->lsa_un.un_rla.rla_options))));
-		ND_PRINT((ndo, ", RLA-Flags [%s]",
+		          GET_BE_U_4(lsap->lsa_un.un_rla.rla_options)));
+		ND_PRINT(", RLA-Flags [%s]",
 		          bittok2str(ospf6_rla_flag_values, "none",
-		          lsap->lsa_un.un_rla.rla_flags)));
+		          GET_U_1(lsap->lsa_un.un_rla.rla_flags)));
 
 		rlp = lsap->lsa_un.un_rla.rla_link;
 		while (lsa_length != 0) {
 			if (lsa_length < sizeof (*rlp))
 				return (1);
 			lsa_length -= sizeof (*rlp);
-			ND_TCHECK(*rlp);
-			switch (rlp->link_type) {
+			ND_TCHECK_SIZE(rlp);
+			switch (GET_U_1(rlp->link_type)) {
 
 			case RLA_TYPE_VIRTUAL:
-				ND_PRINT((ndo, "\n\t      Virtual Link: Neighbor Router-ID %s"
+				ND_PRINT("\n\t      Virtual Link: Neighbor Router-ID %s"
                                        "\n\t      Neighbor Interface-ID %s, Interface %s",
-                                       ipaddr_string(ndo, &rlp->link_nrtid),
-                                       ipaddr_string(ndo, &rlp->link_nifid),
-                                       ipaddr_string(ndo, &rlp->link_ifid)));
+                                       GET_IPADDR_STRING(rlp->link_nrtid),
+                                       GET_IPADDR_STRING(rlp->link_nifid),
+                                       GET_IPADDR_STRING(rlp->link_ifid));
                                 break;
 
 			case RLA_TYPE_ROUTER:
-				ND_PRINT((ndo, "\n\t      Neighbor Router-ID %s"
+				ND_PRINT("\n\t      Neighbor Router-ID %s"
                                        "\n\t      Neighbor Interface-ID %s, Interface %s",
-                                       ipaddr_string(ndo, &rlp->link_nrtid),
-                                       ipaddr_string(ndo, &rlp->link_nifid),
-                                       ipaddr_string(ndo, &rlp->link_ifid)));
+                                       GET_IPADDR_STRING(rlp->link_nrtid),
+                                       GET_IPADDR_STRING(rlp->link_nifid),
+                                       GET_IPADDR_STRING(rlp->link_ifid));
 				break;
 
 			case RLA_TYPE_TRANSIT:
-				ND_PRINT((ndo, "\n\t      Neighbor Network-ID %s"
+				ND_PRINT("\n\t      Neighbor Network-ID %s"
                                        "\n\t      Neighbor Interface-ID %s, Interface %s",
-				    ipaddr_string(ndo, &rlp->link_nrtid),
-				    ipaddr_string(ndo, &rlp->link_nifid),
-				    ipaddr_string(ndo, &rlp->link_ifid)));
+				    GET_IPADDR_STRING(rlp->link_nrtid),
+				    GET_IPADDR_STRING(rlp->link_nifid),
+				    GET_IPADDR_STRING(rlp->link_ifid));
 				break;
 
 			default:
-				ND_PRINT((ndo, "\n\t      Unknown Router Links Type 0x%02x",
-				    rlp->link_type));
+				ND_PRINT("\n\t      Unknown Router Links Type 0x%02x",
+				    GET_U_1(rlp->link_type));
 				return (0);
 			}
-			ND_PRINT((ndo, ", metric %d", EXTRACT_16BITS(&rlp->link_metric)));
+			ND_PRINT(", metric %u", GET_BE_U_2(rlp->link_metric));
 			rlp++;
 		}
 		break;
@@ -544,19 +542,18 @@ ospf6_print_lsa(netdissect_options *ndo,
 		if (lsa_length < sizeof (lsap->lsa_un.un_nla.nla_options))
 			return (1);
 		lsa_length -= sizeof (lsap->lsa_un.un_nla.nla_options);
-		ND_TCHECK(lsap->lsa_un.un_nla.nla_options);
-		ND_PRINT((ndo, "\n\t      Options [%s]",
+		ND_PRINT("\n\t      Options [%s]",
 		          bittok2str(ospf6_option_values, "none",
-		          EXTRACT_32BITS(&lsap->lsa_un.un_nla.nla_options))));
+		          GET_BE_U_4(lsap->lsa_un.un_nla.nla_options)));
 
-		ND_PRINT((ndo, "\n\t      Connected Routers:"));
+		ND_PRINT("\n\t      Connected Routers:");
 		ap = lsap->lsa_un.un_nla.nla_router;
 		while (lsa_length != 0) {
 			if (lsa_length < sizeof (*ap))
 				return (1);
 			lsa_length -= sizeof (*ap);
-			ND_TCHECK(*ap);
-			ND_PRINT((ndo, "\n\t\t%s", ipaddr_string(ndo, ap)));
+			ND_TCHECK_SIZE(ap);
+			ND_PRINT("\n\t\t%s", GET_IPADDR_STRING(*ap));
 			++ap;
 		}
 		break;
@@ -565,15 +562,19 @@ ospf6_print_lsa(netdissect_options *ndo,
 		if (lsa_length < sizeof (lsap->lsa_un.un_inter_ap.inter_ap_metric))
 			return (1);
 		lsa_length -= sizeof (lsap->lsa_un.un_inter_ap.inter_ap_metric);
-		ND_TCHECK(lsap->lsa_un.un_inter_ap.inter_ap_metric);
-		ND_PRINT((ndo, ", metric %u",
-			EXTRACT_32BITS(&lsap->lsa_un.un_inter_ap.inter_ap_metric) & SLA_MASK_METRIC));
+		ND_PRINT(", metric %u",
+			GET_BE_U_4(lsap->lsa_un.un_inter_ap.inter_ap_metric) & SLA_MASK_METRIC);
 
 		tptr = (const uint8_t *)lsap->lsa_un.un_inter_ap.inter_ap_prefix;
 		while (lsa_length != 0) {
 			bytelen = ospf6_print_lsaprefix(ndo, tptr, lsa_length);
 			if (bytelen < 0)
 				goto trunc;
+			/*
+			 * ospf6_print_lsaprefix() will return -1 if
+			 * the length is too high, so this will not
+			 * underflow.
+			 */
 			lsa_length -= bytelen;
 			tptr += bytelen;
 		}
@@ -583,52 +584,50 @@ ospf6_print_lsa(netdissect_options *ndo,
 		if (lsa_length < sizeof (lsap->lsa_un.un_asla.asla_metric))
 			return (1);
 		lsa_length -= sizeof (lsap->lsa_un.un_asla.asla_metric);
-		ND_TCHECK(lsap->lsa_un.un_asla.asla_metric);
-		flags32 = EXTRACT_32BITS(&lsap->lsa_un.un_asla.asla_metric);
-		ND_PRINT((ndo, "\n\t     Flags [%s]",
-		          bittok2str(ospf6_asla_flag_values, "none", flags32)));
-		ND_PRINT((ndo, " metric %u",
-		       EXTRACT_32BITS(&lsap->lsa_un.un_asla.asla_metric) &
-		       ASLA_MASK_METRIC));
+		flags32 = GET_BE_U_4(lsap->lsa_un.un_asla.asla_metric);
+		ND_PRINT("\n\t     Flags [%s]",
+		          bittok2str(ospf6_asla_flag_values, "none", flags32));
+		ND_PRINT(" metric %u",
+		       GET_BE_U_4(lsap->lsa_un.un_asla.asla_metric) &
+		       ASLA_MASK_METRIC);
 
 		tptr = (const uint8_t *)lsap->lsa_un.un_asla.asla_prefix;
 		lsapp = (const struct lsa6_prefix *)tptr;
 		bytelen = ospf6_print_lsaprefix(ndo, tptr, lsa_length);
 		if (bytelen < 0)
 			goto trunc;
+		/*
+		 * ospf6_print_lsaprefix() will return -1 if
+		 * the length is too high, so this will not
+		 * underflow.
+		 */
 		lsa_length -= bytelen;
 		tptr += bytelen;
 
 		if ((flags32 & ASLA_FLAG_FWDADDR) != 0) {
-			const struct in6_addr *fwdaddr6;
-
-			fwdaddr6 = (const struct in6_addr *)tptr;
-			if (lsa_length < sizeof (*fwdaddr6))
+			if (lsa_length < sizeof (nd_ipv6))
 				return (1);
-			lsa_length -= sizeof (*fwdaddr6);
-			ND_TCHECK(*fwdaddr6);
-			ND_PRINT((ndo, " forward %s",
-			       ip6addr_string(ndo, fwdaddr6)));
-			tptr += sizeof(*fwdaddr6);
+			lsa_length -= sizeof (nd_ipv6);
+			ND_PRINT(" forward %s",
+				 GET_IP6ADDR_STRING(tptr));
+			tptr += sizeof(nd_ipv6);
 		}
 
 		if ((flags32 & ASLA_FLAG_ROUTETAG) != 0) {
 			if (lsa_length < sizeof (uint32_t))
 				return (1);
 			lsa_length -= sizeof (uint32_t);
-			ND_TCHECK(*(const uint32_t *)tptr);
-			ND_PRINT((ndo, " tag %s",
-			       ipaddr_string(ndo, (const uint32_t *)tptr)));
+			ND_PRINT(" tag %s",
+			       GET_IPADDR_STRING(tptr));
 			tptr += sizeof(uint32_t);
 		}
 
-		if (lsapp->lsa_p_metric) {
+		if (GET_U_1(lsapp->lsa_p_metric)) {
 			if (lsa_length < sizeof (uint32_t))
 				return (1);
 			lsa_length -= sizeof (uint32_t);
-			ND_TCHECK(*(const uint32_t *)tptr);
-			ND_PRINT((ndo, " RefLSID: %s",
-			       ipaddr_string(ndo, (const uint32_t *)tptr)));
+			ND_PRINT(" RefLSID: %s",
+			       GET_IPADDR_STRING(tptr));
 			tptr += sizeof(uint32_t);
 		}
 		break;
@@ -639,20 +638,19 @@ ospf6_print_lsa(netdissect_options *ndo,
 		if (lsa_length < sizeof (llsap->llsa_priandopt))
 			return (1);
 		lsa_length -= sizeof (llsap->llsa_priandopt);
-		ND_TCHECK(llsap->llsa_priandopt);
-		ND_PRINT((ndo, "\n\t      Options [%s]",
+		ND_TCHECK_SIZE(&llsap->llsa_priandopt);
+		ND_PRINT("\n\t      Options [%s]",
 		          bittok2str(ospf6_option_values, "none",
-		          EXTRACT_32BITS(&llsap->llsa_options))));
+		          GET_BE_U_4(llsap->llsa_options)));
 
 		if (lsa_length < sizeof (llsap->llsa_lladdr) + sizeof (llsap->llsa_nprefix))
 			return (1);
 		lsa_length -= sizeof (llsap->llsa_lladdr) + sizeof (llsap->llsa_nprefix);
-                ND_TCHECK(llsap->llsa_nprefix);
-                prefixes = EXTRACT_32BITS(&llsap->llsa_nprefix);
-		ND_PRINT((ndo, "\n\t      Priority %d, Link-local address %s, Prefixes %d:",
-                       llsap->llsa_priority,
-                       ip6addr_string(ndo, &llsap->llsa_lladdr),
-                       prefixes));
+                prefixes = GET_BE_U_4(llsap->llsa_nprefix);
+		ND_PRINT("\n\t      Priority %u, Link-local address %s, Prefixes %u:",
+                       GET_U_1(llsap->llsa_priority),
+                       GET_IP6ADDR_STRING(llsap->llsa_lladdr),
+                       prefixes);
 
 		tptr = (const uint8_t *)llsap->llsa_prefix;
 		while (prefixes > 0) {
@@ -660,6 +658,11 @@ ospf6_print_lsa(netdissect_options *ndo,
 			if (bytelen < 0)
 				goto trunc;
 			prefixes--;
+			/*
+			 * ospf6_print_lsaprefix() will return -1 if
+			 * the length is too high, so this will not
+			 * underflow.
+			 */
 			lsa_length -= bytelen;
 			tptr += bytelen;
 		}
@@ -670,17 +673,16 @@ ospf6_print_lsa(netdissect_options *ndo,
 		if (lsa_length < sizeof (lsap->lsa_un.un_intra_ap.intra_ap_rtid))
 			return (1);
 		lsa_length -= sizeof (lsap->lsa_un.un_intra_ap.intra_ap_rtid);
-		ND_TCHECK(lsap->lsa_un.un_intra_ap.intra_ap_rtid);
+		ND_TCHECK_4(lsap->lsa_un.un_intra_ap.intra_ap_rtid);
 		ospf6_print_ls_type(ndo,
-			EXTRACT_16BITS(&lsap->lsa_un.un_intra_ap.intra_ap_lstype),
+			GET_BE_U_2(lsap->lsa_un.un_intra_ap.intra_ap_lstype),
 			&lsap->lsa_un.un_intra_ap.intra_ap_lsid);
 
 		if (lsa_length < sizeof (lsap->lsa_un.un_intra_ap.intra_ap_nprefix))
 			return (1);
 		lsa_length -= sizeof (lsap->lsa_un.un_intra_ap.intra_ap_nprefix);
-		ND_TCHECK(lsap->lsa_un.un_intra_ap.intra_ap_nprefix);
-                prefixes = EXTRACT_16BITS(&lsap->lsa_un.un_intra_ap.intra_ap_nprefix);
-		ND_PRINT((ndo, "\n\t      Prefixes %d:", prefixes));
+                prefixes = GET_BE_U_2(lsap->lsa_un.un_intra_ap.intra_ap_nprefix);
+		ND_PRINT("\n\t      Prefixes %u:", prefixes);
 
 		tptr = (const uint8_t *)lsap->lsa_un.un_intra_ap.intra_ap_prefix;
 		while (prefixes > 0) {
@@ -688,19 +690,24 @@ ospf6_print_lsa(netdissect_options *ndo,
 			if (bytelen < 0)
 				goto trunc;
 			prefixes--;
+			/*
+			 * ospf6_print_lsaprefix() will return -1 if
+			 * the length is too high, so this will not
+			 * underflow.
+			 */
 			lsa_length -= bytelen;
 			tptr += bytelen;
 		}
 		break;
 
         case LS_TYPE_GRACE | LS_SCOPE_LINKLOCAL:
-                if (ospf_print_grace_lsa(ndo, tptr, lsa_length) == -1) {
+                if (ospf_grace_lsa_print(ndo, tptr, lsa_length) == -1) {
                     return 1;
                 }
                 break;
 
         case LS_TYPE_INTRA_ATE | LS_SCOPE_LINKLOCAL:
-                if (ospf_print_te_lsa(ndo, tptr, lsa_length) == -1) {
+                if (ospf_te_lsa_print(ndo, tptr, lsa_length) == -1) {
                     return 1;
                 }
                 break;
@@ -721,46 +728,42 @@ trunc:
 
 static int
 ospf6_decode_v3(netdissect_options *ndo,
-                register const struct ospf6hdr *op,
-                register const u_char *dataend)
+                const struct ospf6hdr *op,
+                const u_char *dataend)
 {
-	register const rtrid_t *ap;
-	register const struct lsr6 *lsrp;
-	register const struct lsa6_hdr *lshp;
-	register const struct lsa6 *lsap;
-	register int i;
+	const rtrid_t *ap;
+	const struct lsr6 *lsrp;
+	const struct lsa6_hdr *lshp;
+	const struct lsa6 *lsap;
+	int i;
 
-	switch (op->ospf6_type) {
+	switch (GET_U_1(op->ospf6_type)) {
 
 	case OSPF_TYPE_HELLO: {
-		register const struct hello6 *hellop = (const struct hello6 *)((const uint8_t *)op + OSPF6HDR_LEN);
+		const struct hello6 *hellop = (const struct hello6 *)((const uint8_t *)op + OSPF6HDR_LEN);
 
-		ND_TCHECK_32BITS(&hellop->hello_options);
-		ND_PRINT((ndo, "\n\tOptions [%s]",
+		ND_PRINT("\n\tOptions [%s]",
 		          bittok2str(ospf6_option_values, "none",
-		          EXTRACT_32BITS(&hellop->hello_options))));
+		          GET_BE_U_4(hellop->hello_options)));
 
-		ND_TCHECK(hellop->hello_deadint);
-		ND_PRINT((ndo, "\n\t  Hello Timer %us, Dead Timer %us, Interface-ID %s, Priority %u",
-		          EXTRACT_16BITS(&hellop->hello_helloint),
-		          EXTRACT_16BITS(&hellop->hello_deadint),
-		          ipaddr_string(ndo, &hellop->hello_ifid),
-		          hellop->hello_priority));
+		ND_PRINT("\n\t  Hello Timer %us, Dead Timer %us, Interface-ID %s, Priority %u",
+		          GET_BE_U_2(hellop->hello_helloint),
+		          GET_BE_U_2(hellop->hello_deadint),
+		          GET_IPADDR_STRING(hellop->hello_ifid),
+		          GET_U_1(hellop->hello_priority));
 
-		ND_TCHECK(hellop->hello_dr);
-		if (EXTRACT_32BITS(&hellop->hello_dr) != 0)
-			ND_PRINT((ndo, "\n\t  Designated Router %s",
-			    ipaddr_string(ndo, &hellop->hello_dr)));
-		ND_TCHECK(hellop->hello_bdr);
-		if (EXTRACT_32BITS(&hellop->hello_bdr) != 0)
-			ND_PRINT((ndo, ", Backup Designated Router %s",
-			    ipaddr_string(ndo, &hellop->hello_bdr)));
+		if (GET_BE_U_4(hellop->hello_dr) != 0)
+			ND_PRINT("\n\t  Designated Router %s",
+			    GET_IPADDR_STRING(hellop->hello_dr));
+		if (GET_BE_U_4(hellop->hello_bdr) != 0)
+			ND_PRINT(", Backup Designated Router %s",
+			    GET_IPADDR_STRING(hellop->hello_bdr));
 		if (ndo->ndo_vflag > 1) {
-			ND_PRINT((ndo, "\n\t  Neighbor List:"));
+			ND_PRINT("\n\t  Neighbor List:");
 			ap = hellop->hello_neighbor;
 			while ((const u_char *)ap < dataend) {
-				ND_TCHECK(*ap);
-				ND_PRINT((ndo, "\n\t    %s", ipaddr_string(ndo, ap)));
+				ND_TCHECK_SIZE(ap);
+				ND_PRINT("\n\t    %s", GET_IPADDR_STRING(*ap));
 				++ap;
 			}
 		}
@@ -768,20 +771,17 @@ ospf6_decode_v3(netdissect_options *ndo,
 	}
 
 	case OSPF_TYPE_DD: {
-		register const struct dd6 *ddp = (const struct dd6 *)((const uint8_t *)op + OSPF6HDR_LEN);
+		const struct dd6 *ddp = (const struct dd6 *)((const uint8_t *)op + OSPF6HDR_LEN);
 
-		ND_TCHECK(ddp->db_options);
-		ND_PRINT((ndo, "\n\tOptions [%s]",
+		ND_PRINT("\n\tOptions [%s]",
 		          bittok2str(ospf6_option_values, "none",
-		          EXTRACT_32BITS(&ddp->db_options))));
-		ND_TCHECK(ddp->db_flags);
-		ND_PRINT((ndo, ", DD Flags [%s]",
-		          bittok2str(ospf6_dd_flag_values,"none",ddp->db_flags)));
+		          GET_BE_U_4(ddp->db_options)));
+		ND_PRINT(", DD Flags [%s]",
+		          bittok2str(ospf6_dd_flag_values,"none",GET_U_1(ddp->db_flags)));
 
-		ND_TCHECK(ddp->db_seq);
-		ND_PRINT((ndo, ", MTU %u, DD-Sequence 0x%08x",
-                       EXTRACT_16BITS(&ddp->db_mtu),
-                       EXTRACT_32BITS(&ddp->db_seq)));
+		ND_PRINT(", MTU %u, DD-Sequence 0x%08x",
+                       GET_BE_U_2(ddp->db_mtu),
+                       GET_BE_U_4(ddp->db_seq));
 		if (ndo->ndo_vflag > 1) {
 			/* Print all the LS adv's */
 			lshp = ddp->db_lshdr;
@@ -797,10 +797,11 @@ ospf6_decode_v3(netdissect_options *ndo,
 		if (ndo->ndo_vflag > 1) {
 			lsrp = (const struct lsr6 *)((const uint8_t *)op + OSPF6HDR_LEN);
 			while ((const u_char *)lsrp < dataend) {
-				ND_TCHECK(*lsrp);
-				ND_PRINT((ndo, "\n\t  Advertising Router %s",
-				          ipaddr_string(ndo, &lsrp->ls_router)));
-				ospf6_print_ls_type(ndo, EXTRACT_16BITS(&lsrp->ls_type),
+				ND_TCHECK_SIZE(lsrp);
+				ND_PRINT("\n\t  Advertising Router %s",
+				          GET_IPADDR_STRING(lsrp->ls_router));
+				ospf6_print_ls_type(ndo,
+                                                    GET_BE_U_2(lsrp->ls_type),
                                                     &lsrp->ls_stateid);
 				++lsrp;
 			}
@@ -809,16 +810,15 @@ ospf6_decode_v3(netdissect_options *ndo,
 
 	case OSPF_TYPE_LS_UPDATE:
 		if (ndo->ndo_vflag > 1) {
-			register const struct lsu6 *lsup = (const struct lsu6 *)((const uint8_t *)op + OSPF6HDR_LEN);
+			const struct lsu6 *lsup = (const struct lsu6 *)((const uint8_t *)op + OSPF6HDR_LEN);
 
-			ND_TCHECK(lsup->lsu_count);
-			i = EXTRACT_32BITS(&lsup->lsu_count);
+			i = GET_BE_U_4(lsup->lsu_count);
 			lsap = lsup->lsu_lsa;
 			while ((const u_char *)lsap < dataend && i--) {
 				if (ospf6_print_lsa(ndo, lsap, dataend))
 					goto trunc;
 				lsap = (const struct lsa6 *)((const u_char *)lsap +
-				    EXTRACT_16BITS(&lsap->ls_hdr.ls_length));
+				    GET_BE_U_2(lsap->ls_hdr.ls_length));
 			}
 		}
 		break;
@@ -853,18 +853,16 @@ ospf6_print_lls(netdissect_options *ndo,
 	if (len < OSPF_LLS_HDRLEN)
 		goto trunc;
 	/* Checksum */
-	ND_TCHECK2(*cp, 2);
-	ND_PRINT((ndo, "\n\tLLS Checksum 0x%04x", EXTRACT_16BITS(cp)));
+	ND_PRINT("\n\tLLS Checksum 0x%04x", GET_BE_U_2(cp));
 	cp += 2;
 	/* LLS Data Length */
-	ND_TCHECK2(*cp, 2);
-	llsdatalen = EXTRACT_16BITS(cp);
-	ND_PRINT((ndo, ", Data Length %u", llsdatalen));
+	llsdatalen = GET_BE_U_2(cp);
+	ND_PRINT(", Data Length %u", llsdatalen);
 	if (llsdatalen < OSPF_LLS_HDRLEN || llsdatalen > len)
 		goto trunc;
 	cp += 2;
 	/* LLS TLVs */
-	ND_TCHECK2(*cp, llsdatalen - OSPF_LLS_HDRLEN);
+	ND_TCHECK_LEN(cp, llsdatalen - OSPF_LLS_HDRLEN);
 	/* FIXME: code in print-ospf.c can be reused to decode the TLVs */
 
 	return llsdatalen;
@@ -884,33 +882,28 @@ ospf6_decode_at(netdissect_options *ndo,
 	if (len < OSPF6_AT_HDRLEN)
 		goto trunc;
 	/* Authentication Type */
-	ND_TCHECK2(*cp, 2);
-	ND_PRINT((ndo, "\n\tAuthentication Type %s", tok2str(ospf6_auth_type_str, "unknown (0x%04x)", EXTRACT_16BITS(cp))));
+	ND_PRINT("\n\tAuthentication Type %s",
+		 tok2str(ospf6_auth_type_str, "unknown (0x%04x)", GET_BE_U_2(cp)));
 	cp += 2;
 	/* Auth Data Len */
-	ND_TCHECK2(*cp, 2);
-	authdatalen = EXTRACT_16BITS(cp);
-	ND_PRINT((ndo, ", Length %u", authdatalen));
+	authdatalen = GET_BE_U_2(cp);
+	ND_PRINT(", Length %u", authdatalen);
 	if (authdatalen < OSPF6_AT_HDRLEN || authdatalen > len)
 		goto trunc;
 	cp += 2;
 	/* Reserved */
-	ND_TCHECK2(*cp, 2);
 	cp += 2;
 	/* Security Association ID */
-	ND_TCHECK2(*cp, 2);
-	ND_PRINT((ndo, ", SAID %u", EXTRACT_16BITS(cp)));
+	ND_PRINT(", SAID %u", GET_BE_U_2(cp));
 	cp += 2;
 	/* Cryptographic Sequence Number (High-Order 32 Bits) */
-	ND_TCHECK2(*cp, 4);
-	ND_PRINT((ndo, ", CSN 0x%08x", EXTRACT_32BITS(cp)));
+	ND_PRINT(", CSN 0x%08x", GET_BE_U_4(cp));
 	cp += 4;
 	/* Cryptographic Sequence Number (Low-Order 32 Bits) */
-	ND_TCHECK2(*cp, 4);
-	ND_PRINT((ndo, ":%08x", EXTRACT_32BITS(cp)));
+	ND_PRINT(":%08x", GET_BE_U_4(cp));
 	cp += 4;
 	/* Authentication Data */
-	ND_TCHECK2(*cp, authdatalen - OSPF6_AT_HDRLEN);
+	ND_TCHECK_LEN(cp, authdatalen - OSPF6_AT_HDRLEN);
 	if (ndo->ndo_vflag > 1)
 		print_unknown_data(ndo,cp, "\n\tAuthentication Data ", authdatalen - OSPF6_AT_HDRLEN);
 	return 0;
@@ -928,19 +921,19 @@ static int
 ospf6_decode_v3_trailer(netdissect_options *ndo,
                         const struct ospf6hdr *op, const u_char *cp, const unsigned len)
 {
+	uint8_t type;
 	int llslen = 0;
 	int lls_hello = 0;
 	int lls_dd = 0;
 
-	if (op->ospf6_type == OSPF_TYPE_HELLO) {
+	type = GET_U_1(op->ospf6_type);
+	if (type == OSPF_TYPE_HELLO) {
 		const struct hello6 *hellop = (const struct hello6 *)((const uint8_t *)op + OSPF6HDR_LEN);
-		ND_TCHECK(hellop->hello_options);
-		if (EXTRACT_32BITS(&hellop->hello_options) & OSPF6_OPTION_L)
+		if (GET_BE_U_4(hellop->hello_options) & OSPF6_OPTION_L)
 			lls_hello = 1;
-	} else if (op->ospf6_type == OSPF_TYPE_DD) {
+	} else if (type == OSPF_TYPE_DD) {
 		const struct dd6 *ddp = (const struct dd6 *)((const uint8_t *)op + OSPF6HDR_LEN);
-		ND_TCHECK(ddp->db_options);
-		if (EXTRACT_32BITS(&ddp->db_options) & OSPF6_OPTION_L)
+		if (GET_BE_U_4(ddp->db_options) & OSPF6_OPTION_L)
 			lls_dd = 1;
 	}
 	if ((lls_hello || lls_dd) && (llslen = ospf6_print_lls(ndo, cp, len)) < 0)
@@ -953,20 +946,22 @@ trunc:
 
 void
 ospf6_print(netdissect_options *ndo,
-            register const u_char *bp, register u_int length)
+            const u_char *bp, u_int length)
 {
-	register const struct ospf6hdr *op;
-	register const u_char *dataend;
-	register const char *cp;
+	const struct ospf6hdr *op;
+	const u_char *dataend;
+	const char *cp;
 	uint16_t datalen;
 
+	ndo->ndo_protocol = "ospf3";
 	op = (const struct ospf6hdr *)bp;
 
 	/* If the type is valid translate it, or just print the type */
 	/* value.  If it's not valid, say so and return */
-	ND_TCHECK(op->ospf6_type);
-	cp = tok2str(ospf6_type_values, "unknown packet type (%u)", op->ospf6_type);
-	ND_PRINT((ndo, "OSPFv%u, %s, length %d", op->ospf6_version, cp, length));
+	cp = tok2str(ospf6_type_values, "unknown packet type (%u)",
+		     GET_U_1(op->ospf6_type));
+	ND_PRINT("OSPFv%u, %s, length %u", GET_U_1(op->ospf6_version), cp,
+		 length);
 	if (*cp == 'u') {
 		return;
 	}
@@ -976,28 +971,24 @@ ospf6_print(netdissect_options *ndo,
 	}
 
 	/* OSPFv3 data always comes first and optional trailing data may follow. */
-	ND_TCHECK(op->ospf6_len);
-	datalen = EXTRACT_16BITS(&op->ospf6_len);
+	datalen = GET_BE_U_2(op->ospf6_len);
 	if (datalen > length) {
-		ND_PRINT((ndo, " [len %d]", datalen));
+		ND_PRINT(" [len %u]", datalen);
 		return;
 	}
 	dataend = bp + datalen;
 
-	ND_TCHECK(op->ospf6_routerid);
-	ND_PRINT((ndo, "\n\tRouter-ID %s", ipaddr_string(ndo, &op->ospf6_routerid)));
+	ND_PRINT("\n\tRouter-ID %s", GET_IPADDR_STRING(op->ospf6_routerid));
 
-	ND_TCHECK(op->ospf6_areaid);
-	if (EXTRACT_32BITS(&op->ospf6_areaid) != 0)
-		ND_PRINT((ndo, ", Area %s", ipaddr_string(ndo, &op->ospf6_areaid)));
+	if (GET_BE_U_4(op->ospf6_areaid) != 0)
+		ND_PRINT(", Area %s", GET_IPADDR_STRING(op->ospf6_areaid));
 	else
-		ND_PRINT((ndo, ", Backbone Area"));
-	ND_TCHECK(op->ospf6_instanceid);
-	if (op->ospf6_instanceid)
-		ND_PRINT((ndo, ", Instance %u", op->ospf6_instanceid));
+		ND_PRINT(", Backbone Area");
+	if (GET_U_1(op->ospf6_instanceid))
+		ND_PRINT(", Instance %u", GET_U_1(op->ospf6_instanceid));
 
 	/* Do rest according to version.	 */
-	switch (op->ospf6_version) {
+	switch (GET_U_1(op->ospf6_version)) {
 
 	case 3:
 		/* ospf version 3 */
@@ -1009,5 +1000,5 @@ ospf6_print(netdissect_options *ndo,
 
 	return;
 trunc:
-	ND_PRINT((ndo, "%s", tstr));
+	nd_print_trunc(ndo);
 }
