@@ -19,73 +19,97 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
+/* \summary: IPv6 routing header printer */
+
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
-
-#include <string.h>
-
-#include "ip6.h"
+#include "netdissect-stdinc.h"
 
 #include "netdissect.h"
 #include "addrtoname.h"
 #include "extract.h"
 
+#include "ip6.h"
+
 int
-rt6_print(netdissect_options *ndo, register const u_char *bp, const u_char *bp2 _U_)
+rt6_print(netdissect_options *ndo, const u_char *bp, const u_char *bp2 _U_)
 {
-	register const struct ip6_rthdr *dp;
-	register const struct ip6_rthdr0 *dp0;
-	register const u_char *ep;
-	int i, len;
-	register const struct in6_addr *addr;
+	const struct ip6_rthdr *dp;
+	const struct ip6_rthdr0 *dp0;
+	const struct ip6_srh *srh;
+	u_int i, len, type;
+	const u_char *p;
 
+	ndo->ndo_protocol = "rt6";
+
+	nd_print_protocol_caps(ndo);
 	dp = (const struct ip6_rthdr *)bp;
-	len = dp->ip6r_len;
 
-	/* 'ep' points to the end of available data. */
-	ep = ndo->ndo_snapend;
+	len = GET_U_1(dp->ip6r_len);
+	ND_PRINT(" (len=%u", len);	/*)*/
+	type = GET_U_1(dp->ip6r_type);
+	ND_PRINT(", type=%u", type);
+	if (type == IPV6_RTHDR_TYPE_0)
+		ND_PRINT(" [Deprecated]");
+	ND_PRINT(", segleft=%u", GET_U_1(dp->ip6r_segleft));
 
-	ND_TCHECK(dp->ip6r_segleft);
-
-	ND_PRINT((ndo, "srcrt (len=%d", dp->ip6r_len));	/*)*/
-	ND_PRINT((ndo, ", type=%d", dp->ip6r_type));
-	ND_PRINT((ndo, ", segleft=%d", dp->ip6r_segleft));
-
-	switch (dp->ip6r_type) {
+	switch (type) {
 	case IPV6_RTHDR_TYPE_0:
 	case IPV6_RTHDR_TYPE_2:			/* Mobile IPv6 ID-20 */
 		dp0 = (const struct ip6_rthdr0 *)dp;
 
-		ND_TCHECK(dp0->ip6r0_reserved);
-		if (dp0->ip6r0_reserved || ndo->ndo_vflag) {
-			ND_PRINT((ndo, ", rsv=0x%0x",
-			    EXTRACT_32BITS(&dp0->ip6r0_reserved)));
+		if (GET_BE_U_4(dp0->ip6r0_reserved) || ndo->ndo_vflag) {
+			ND_PRINT(", rsv=0x%0x",
+			    GET_BE_U_4(dp0->ip6r0_reserved));
 		}
 
-		if (len % 2 == 1)
-			goto trunc;
+		if (len % 2 == 1) {
+			ND_PRINT(" (invalid length %u)", len);
+			goto invalid;
+		}
 		len >>= 1;
-		addr = &dp0->ip6r0_addr[0];
+		p = (const u_char *) dp0->ip6r0_addr;
 		for (i = 0; i < len; i++) {
-			if ((const u_char *)(addr + 1) > ep)
-				goto trunc;
-
-			ND_PRINT((ndo, ", [%d]%s", i, ip6addr_string(ndo, addr)));
-			addr++;
+			ND_PRINT(", [%u]%s", i, GET_IP6ADDR_STRING(p));
+			p += 16;
 		}
 		/*(*/
-		ND_PRINT((ndo, ") "));
-		return((dp0->ip6r0_len + 1) << 3);
+		ND_PRINT(") ");
+		return((GET_U_1(dp0->ip6r0_len) + 1) << 3);
+		break;
+	case IPV6_RTHDR_TYPE_4:
+		srh = (const struct ip6_srh *)dp;
+		ND_PRINT(", last-entry=%u", GET_U_1(srh->srh_last_ent));
+
+		if (GET_U_1(srh->srh_flags) || ndo->ndo_vflag) {
+			ND_PRINT(", flags=0x%0x",
+				GET_U_1(srh->srh_flags));
+		}
+
+		ND_PRINT(", tag=%x", GET_BE_U_2(srh->srh_tag));
+
+		if (len % 2 == 1) {
+			ND_PRINT(" (invalid length %u)", len);
+			goto invalid;
+		}
+		len >>= 1;
+		p  = (const u_char *) srh->srh_segments;
+		for (i = 0; i < len; i++) {
+			ND_PRINT(", [%u]%s", i, GET_IP6ADDR_STRING(p));
+			p += 16;
+		}
+		/*(*/
+		ND_PRINT(") ");
+		return((GET_U_1(srh->srh_len) + 1) << 3);
 		break;
 	default:
-		goto trunc;
-		break;
+		ND_PRINT(" (unknown type)");
+		goto invalid;
 	}
 
- trunc:
-	ND_PRINT((ndo, "[|srcrt]"));
+invalid:
+	nd_print_invalid(ndo);
 	return -1;
 }
