@@ -1,4 +1,3 @@
-/* @(#) $Header: /tcpdump/master/tcpdump/rpc_auth.h,v 1.2 2005-04-27 21:43:48 guy Exp $ (LBL) */
 /*
  * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
  * unrestricted use provided that this legend is included on all tape
@@ -66,14 +65,7 @@ enum sunrpc_auth_stat {
  * Authentication info.  Opaque to client.
  */
 struct sunrpc_opaque_auth {
-	u_int32_t oa_flavor;		/* flavor of auth */
-	u_int32_t oa_len;		/* length of opaque body */
+	nd_uint32_t oa_flavor;		/* flavor of auth */
+	nd_uint32_t oa_len;		/* length of opaque body */
 	/* zero or more bytes of body */
 };
-
-#define SUNRPC_AUTH_NONE	0	/* no authentication */
-#define	SUNRPC_AUTH_NULL	0	/* backward compatibility */
-#define	SUNRPC_AUTH_UNIX	1	/* unix style (uid, gids) */
-#define	SUNRPC_AUTH_SYS		1	/* forward compatibility */
-#define	SUNRPC_AUTH_SHORT	2	/* short hand unix style */
-#define SUNRPC_AUTH_DES		3	/* des style (encrypted timestamps) */
